unit DMain;

interface

uses
  System.SysUtils, System.Classes, System.Messaging, FireDAC.Stan.Intf,
  FireDAC.Stan.Option, FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS,
  FireDAC.Phys.Intf, FireDAC.DApt.Intf, Data.DB, FireDAC.Comp.DataSet,
  FireDAC.Comp.Client, BothSides.Messages;

type
  TDMMain = class(TDataModule)
    tblContacts: TFDMemTable;
    tblContactsID: TAutoIncField;
    tblContactsNAME: TWideStringField;
    tblContactsEMAIL: TWideStringField;
    procedure DataModuleCreate(Sender: TObject);
  private
    FMessageManager: TMessageManager;
    { Private declarations }
    function LoadContacts : TContactArray;
    procedure SubscribeMessages;
    procedure AddNewContact( ContactName, ContactEmail : String;out Id : Integer );
    procedure UpdateContact( ID : Integer;ContactName, ContactEmail : String );
  public
    { Public declarations }
    property MsgManager : TMessageManager read FMessageManager;
  end;

var
  DMMain: TDMMain;

implementation

uses BothSides.Entities, BothSides.Lists;

{%CLASSGROUP 'System.Classes.TPersistent'}

{$R *.dfm}

procedure TDMMain.AddNewContact(ContactName, ContactEmail: String;out Id : Integer);
begin
  tblContacts.Append;
  tblContactsNAME.Value := ContactName;
  tblContactsEMAIL.Value := ContactEmail;
  tblContacts.Post;
  Id := tblContactsID.Value;
end;

procedure TDMMain.DataModuleCreate(Sender: TObject);
begin
  FMessageManager := TMessageManager.DefaultManager;
  SubscribeMessages;
  tblContacts.Active := True;
  tblContacts.Append;
  tblContactsNAME.Value := 'David Millington';
  tblContactsEMAIL.Value := 'david.millington@embarcadero.com';
  tblContacts.Post;
  tblContacts.Append;
  tblContactsNAME.Value := 'Marco Cantu`';
  tblContactsEMAIL.Value := 'marco.cantu@embarcadero.com';
  tblContacts.Post;
end;

function TDMMain.LoadContacts: TContactArray;
var AList : TContactList;
    AContact : TContact;
begin
  AList := TContactList.Create;
  tblContacts.First;
  while Not tblContacts.Eof do
    begin
      AContact.Id := tblContactsID.Value;
      AContact.Name := tblContactsNAME.Value;
      AContact.Email := tblContactsEMAIL.Value;
      AList.Add( AContact );
      tblContacts.Next;
    end;
  Result := AList.ToArray;
end;

procedure TDMMain.SubscribeMessages;
begin
  FMessageManager.SubscribeToMessage( TUILoadContactsRequest, procedure( const Sender : TObject;const Message : TMessage )
                                                              var ContactListMessage : TUILoadContacts;
                                                              begin
                                                                ContactListMessage := TUILoadContacts.Create( LoadContacts );
                                                                FMessageManager.SendMessage( Self,ContactListMessage );
                                                              end );
  FMessageManager.SubscribeToMessage( TNewContactAdd, procedure( const Sender : TObject;const Msg: TMessage )
                                                      var ContactAdd: TNewContactAdd;
                                                          ContactAdded: TNewContactAdded;
                                                          Contact : TContact;
                                                      begin
                                                        ContactAdd := TNewContactAdd( Msg );
                                                        Contact := ContactAdd.Value;
                                                        AddNewContact( ContactAdd.Value.Name, ContactAdd.Value.Email,Contact.Id );
                                                        FMessageManager.SendMessage( Self, TNewContactAdded.Create( Contact ));
                                                      end);
  FMessageManager.SubscribeToMessage( TContactChanged, procedure( const Sender: TObject;const Msg: TMessage )
                                                       var ContactChanged : TContactChanged;
                                                       begin
                                                         ContactChanged := TContactChanged( Msg );
                                                         UpdateContact( ContactChanged.Value.Id,ContactChanged.Value.Name, ContactChanged.Value.Email );
                                                       end);
end;

procedure TDMMain.UpdateContact(ID: Integer; ContactName, ContactEmail: String);
begin
  if tblContacts.Locate( 'ID',ID ) then
    begin
      tblContacts.Edit;
      tblContactsNAME.Value := ContactName;
      tblContactsEMAIL.Value := ContactEmail;
      tblContacts.Post;
    end;
end;

end.
