unit Generic.UI;

interface

uses  Classes, Generics.Collections, Generics.Defaults, System.Messaging;

type

  TBaseEvents<TUIControl> = class
  strict private
    FOwner : TUIControl;
    FMsgManager: TMessageManager;
  private
  strict protected
    procedure HandleClick( Sender : TObject );virtual;abstract;
    procedure HandleDblClick( Sender : TObject );virtual;abstract;
    property Owner: TUIControl read FOwner;
  public
    constructor Create( AOwner : TUIControl );virtual;
    procedure AssignEvents;virtual;abstract;
    property MsgManager : TMessageManager read FMsgManager write FMsgManager;
  end;

  TGenericGrid<TGridControl;TData> = class
  strict private
    FDataList: TList<TData>;
    FEvents : TBaseEvents<TGridControl>;
    FControl : TGridControl;

  strict protected
    procedure CreateColumns;virtual;abstract;
    function CreateEvents: TBaseEvents<TGridControl>;virtual;abstract;
    procedure InternalFillList;virtual;abstract;
    property GridControl : TGridControl read FControl;
    function GetID( AData: TData ) : Integer;virtual;abstract;
  public
    constructor Create( AGrid: TGridControl );virtual;
    destructor Destroy; override;
    procedure PrepareForUserInteraction;
    procedure BeginUpdate;virtual;abstract;
    procedure EndUpdate;virtual;abstract;
    procedure Clear;virtual;abstract;
    procedure FillList;
    property DataList : TList<TData> read FDataList;
  end;

implementation


{ TBaseEvents<TUIControl> }

constructor TBaseEvents<TUIControl>.Create(AOwner: TUIControl);
begin
  inherited Create;
  FOwner := AOwner;
end;

{ TGenericGrid<TGridControl, TData> }

constructor TGenericGrid<TGridControl, TData>.Create(AGrid: TGridControl);
begin
  inherited Create;
  FDataList := TList<TData>.Create;
  FControl := AGrid;
end;

destructor TGenericGrid<TGridControl, TData>.Destroy;
begin
  FDataList.Free;
  FEvents.Free;
  inherited;
end;

procedure TGenericGrid<TGridControl, TData>.FillList;
begin
  //BeginUpdate;
  //Clear;
  InternalFillList;
  //EndUpdate;
end;

procedure TGenericGrid<TGridControl, TData>.PrepareForUserInteraction;
begin
  FEvents := CreateEvents;
  FEvents.AssignEvents;
end;

end.
