unit FMX.MainCode;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes,
  System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.StdCtrls,
  FMX.Controls.Presentation, FMX.MultiView, FMX.UI, System.Rtti, FMX.Grid.Style,
  FMX.ScrollBox, FMX.Grid, BothSides.Messages, frameContact;

type
  TMainFrm = class(TForm)
    mvContacts: TMultiView;
    btnDrawer: TButton;
    grpContacts: TGroupBox;
    pnlClient: TPanel;
    grdContacts: TStringGrid;
    ContactFrame1: TContactFrame;
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FListViewUI: TContactListUI;
    procedure SubscribeMessages;
    procedure LoadList( Contacts : TContactArray );
  public
    { Public declarations }
  end;

var
  MainFrm: TMainFrm;

implementation

uses DMain, BothSides.Entities,  System.Messaging;

{$R *.fmx}
{$R *.LgXhdpiTb.fmx ANDROID}

procedure TMainFrm.FormShow(Sender: TObject);
begin
  FListViewUI := TContactListUI.Create( grdContacts );
  SubscribeMessages;
  FListViewUI.PrepareForUserInteraction;
  DMMain.MsgManager.SendMessage( Self, TUILoadContactsRequest.Create );
end;

procedure TMainFrm.LoadList(Contacts: TContactArray);
var Contact : TContact;
begin
  for Contact in Contacts do
    begin
      FListViewUI.DataList.Add( Contact );
    end;
  FListViewUI.FillList;
end;

procedure TMainFrm.SubscribeMessages;
begin
  DMMain.MsgManager.SubscribeToMessage( TUILoadContacts, procedure( const Sender: TObject;const Msg: TMessage )
                                                        var ContactListUI : TUILoadContacts;
                                                        begin
                                                          ContactListUI := TUILoadContacts(Msg);
                                                          LoadList( ContactListUI.Value );
                                                        end);
  DMMain.MsgManager.SubscribeToMessage( TNewContactAdded,procedure( const Sender: TObject;const Msg: TMessage )
                                                         var NewContactAdded : TNewContactAdded;
                                                             Contact : TContact;
                                                         begin
                                                           NewContactAdded := TNewContactAdded( Msg );
                                                           Contact := NewContactAdded.Value;
                                                           FListViewUI.DataList.Add( Contact );
                                                           FListViewUI.FillList;
                                                         end);
  DMMain.MsgManager.SubscribeToMessage( TContactChanged, procedure( const Sender: TObject;const Msg : TMessage )
                                                         var ContactChanged : TContactChanged;
                                                         begin
                                                           ContactChanged := TContactChanged( Msg );
                                                           FListViewUI.Contacts[ ContactChanged.Value.Id ] := ContactChanged.Value;
                                                           FListViewUI.FillList;
                                                         end);
end;

end.
