unit FMX.UI;

interface

uses Generic.UI, FMX.Grid, BothSides.Entities,
     BothSides.Messages;

Type

  TContactListEvents = class( TBaseEvents<TStringGrid> )
  strict protected
    procedure HandleStringGridDblClick(const Column: TColumn;  const Row: Integer);
    function ContactFromRow( ARow: Integer ) : TContact;
    procedure HandleClick( Sender : TObject );override;
    procedure HandleDblClick( Sender : TObject );override;
  public
    constructor Create( AOwner : TStringGrid );override;
    procedure AssignEvents;override;
  end;

  TContactListUI = class( TGenericGrid<TStringGrid,TContact> )
  private
    function GetContacts(ID: Integer): TContact;
    procedure SetContacts(ID: Integer; const Value: TContact);
  strict protected
    procedure CreateColumns;override;
    procedure InternalFillList;override;
    function CreateEvents: TBaseEvents<TStringGrid>;override;
    function GetID( AData: TContact ) : Integer;override;
  public
    constructor Create( AGrid: TStringGrid );override;
    procedure BeginUpdate;override;
    procedure EndUpdate;override;
    procedure Clear;override;
    property Contacts[ ID : Integer ] : TContact read GetContacts write SetContacts;
  end;

implementation

uses Math, SysUtils, DMain;

{ TContactListUI }

procedure TContactListUI.BeginUpdate;
begin
  inherited;
  GridControl.BeginUpdate;
end;

procedure TContactListUI.Clear;
begin
  inherited;
  GridControl.RowCount := 0;
end;

constructor TContactListUI.Create(AGrid: TStringGrid);
begin
  inherited;
  CreateColumns;
end;

procedure TContactListUI.CreateColumns;
var Column : TColumn;
begin
  inherited;
  Column := TColumn.Create( nil );
  Column.Width := Floor(GridControl.Width) div 2;
  Column.Header := 'Name';
  Column.Parent := GridControl;
  Column := TColumn.Create( nil );
  Column.Width := Floor(GridControl.Width) div 2;
  Column.Header := 'Email';
  Column.Parent := GridControl;
  Column := TColumn.Create( nil );
  Column.Header := 'Hidden';
  Column.Visible := False;
  Column.Parent := GridControl;
end;

function TContactListUI.CreateEvents: TBaseEvents<TStringGrid>;
begin
  Result := TContactListEvents.Create( GridControl );
end;

procedure TContactListUI.EndUpdate;
begin
  inherited;
  GridControl.EndUpdate;
end;

function TContactListUI.GetContacts(ID: Integer): TContact;
var Contact : TContact;
begin
  for Contact in DataList do
    begin
      if Contact.Id = ID then
        break;
    end;
  Result := Contact;
end;

function TContactListUI.GetID(AData: TContact): Integer;
begin
  Result := AData.Id;
end;

procedure TContactListUI.InternalFillList;
var Contact : TContact;
    RowIndex : Integer;
begin
  inherited;
  BeginUpdate;
  Clear;
  GridControl.RowCount := DataList.Count;
  try
  RowIndex := 0;
  for Contact in DataList do
    begin
      GridControl.Cells[0,RowIndex] := Contact.Name;
      GridControl.Cells[ 1,RowIndex ] := Contact.Email;
      GridControl.Cells[ 2, RowIndex ] := Contact.Id.ToString;
      Inc( RowIndex );
    end;
  finally
    EndUpdate;
  end;
end;

procedure TContactListUI.SetContacts(ID: Integer; const Value: TContact);
var Contact : TContact;
    ContactIdx : Integer;
begin
  ContactIdx := -1;
  for Contact in DataList do
    begin
      if Contact.Id = ID then
        begin
          ContactIdx := DataList.IndexOf( Contact );
        end;
    end;
  if ContactIdx >= 0 then
    begin
      DataList.Items[ ContactIdx ] := Value;
    end;
end;

{ TContactListEvents }

procedure TContactListEvents.AssignEvents;
begin
  inherited;
  Owner.OnCellDblClick := HandleStringGridDblClick;
end;

function TContactListEvents.ContactFromRow( ARow: Integer ): TContact;
begin
  Result.Id := Owner.Cells[ 2,ARow ].ToInteger;
  Result.Name := Owner.Cells[ 0,ARow ];
  Result.Email := Owner.Cells[ 1,ARow];
end;

constructor TContactListEvents.Create(AOwner: TStringGrid);
begin
  inherited;
  MsgManager := DMMain.MsgManager;
end;

procedure TContactListEvents.HandleClick(Sender: TObject);
begin
  inherited;

end;

procedure TContactListEvents.HandleDblClick(Sender: TObject);
begin
  inherited;

end;

procedure TContactListEvents.HandleStringGridDblClick(const Column: TColumn;
  const Row: Integer);
var ContactDblClicked : TContactDblClicked;
begin
  ContactDblClicked := TContactDblClicked.Create( ContactFromRow( Row ) );
  MsgManager.SendMessage( Self, ContactDblClicked );
end;

end.
