unit VCL.UI;

interface

uses Generic.UI, Controls, ComCtrls, BothSides.Entities;

Type

  TUIBaseEvents<TUIControl: TWinControl> = class( TBaseEvents<TUIControl> )
  public

  end;

  TListViewGrid<TData> = class( TGenericGrid<TListView,TData> )
  public
    procedure BeginUpdate;override;
    procedure EndUpdate;override;
    procedure Clear;override;
  end;

  TContactListViewEvents = class( TUIBaseEvents<TListView> )
  strict protected
    procedure HandleClick( Sender : TObject );override;
    procedure HandleDblClick( Sender : TObject );override;
    procedure HandleChange(Sender: TObject; Item: TListItem; Change: TItemChange);
  public
    constructor Create( AOwner : TListView );override;
    procedure AssignEvents;override;
  end;

  TContactListViewGrid = class( TListViewGrid<TContact> )
  private
    function GetContacts(ID: Integer): TContact;
    procedure SetContacts(ID: Integer; const Value: TContact);
  strict protected
    procedure CreateColumns;override;
    function CreateEvents: TBaseEvents<TListView>;override;
    procedure InternalFillList;override;
    function GetID( AData: TContact ) : Integer;override;
  public
    constructor Create( AGrid: TListView );override;
    property Contacts[ ID : Integer ] : TContact read GetContacts write SetContacts;
  end;

implementation

uses System.Messaging, BothSides.Messages, DMain;

{ TListViewGrid<TData> }

procedure TListViewGrid<TData>.BeginUpdate;
begin
  inherited;
   GridControl.Items.BeginUpdate;
end;

procedure TListViewGrid<TData>.Clear;
begin
  inherited;
  GridControl.Items.Clear;
end;

procedure TListViewGrid<TData>.EndUpdate;
begin
  inherited;
  GridControl.Items.EndUpdate;
end;

{ TContactListViewGrid }

constructor TContactListViewGrid.Create(AGrid: TListView);
begin
  inherited;
  CreateColumns;
end;

procedure TContactListViewGrid.CreateColumns;
var LC : TListColumn;
begin
  inherited;
  LC := GridControl.Columns.Add;
  LC.Caption := 'Name';
  LC.Width := GridControl.ClientWidth div 2;
  LC := GridControl.Columns.Add;
  LC.Caption := 'Email';
  LC.Width := GridControl.ClientWidth div 2;
end;

function TContactListViewGrid.CreateEvents: TBaseEvents<TListView>;
begin
  Result := TContactListViewEvents.Create( GridControl );
  Result.AssignEvents;
end;

function TContactListViewGrid.GetContacts(ID: Integer): TContact;
var Contact : TContact;
begin
  for Contact in DataList do
    begin
      if Contact.Id = ID then
        break;
    end;
  Result := Contact;
end;

function TContactListViewGrid.GetID(AData: TContact): Integer;
begin
  Result := AData.Id;
end;

procedure TContactListViewGrid.InternalFillList;
var Contact : TContact;
    LI : TListItem;
    LIContact : PContact;
begin
  inherited;
  GridControl.Items.BeginUpdate;
  GridControl.Items.Clear;
  try
    for Contact in DataList do
      begin
        New( LIContact );
        LIContact^ := Contact;
        LI := GridControl.Items.Add;
        LI.Caption := Contact.Name;
        LI.SubItems.Add( Contact.Email );
        LI.Data := LIContact;
      end;
  finally
    GridControl.Items.EndUpdate;
  end;
end;

procedure TContactListViewGrid.SetContacts(ID: Integer; const Value: TContact);
var Contact : TContact;
    ContactIdx : Integer;
begin
  ContactIdx := -1;
  for Contact in DataList do
    begin
      if Contact.Id = ID then
        begin
          ContactIdx := DataList.IndexOf( Contact );
        end;
    end;
  if ContactIdx >= 0 then
    begin
      DataList.Items[ ContactIdx ] := Value;
    end;
end;

{ TContactListViewEvents }

procedure TContactListViewEvents.AssignEvents;
begin
  inherited;
  Owner.OnClick := HandleClick;
  Owner.OnDblClick := HandleDblClick;
  Owner.OnChange := HandleChange;
end;

constructor TContactListViewEvents.Create(AOwner: TListView);
begin
  inherited;
  MsgManager := DMMain.MsgManager;
end;

procedure TContactListViewEvents.HandleChange(Sender: TObject; Item: TListItem; Change: TItemChange);
begin

end;

procedure TContactListViewEvents.HandleClick( Sender : TObject );
begin
  inherited;

end;

procedure TContactListViewEvents.HandleDblClick( Sender : TObject );
var ContactDblClicked : TContactDblClicked;
    Contact : TContact;
    SelectedContact : TListItem;
begin
  inherited;
  SelectedContact := Owner.Selected;
  if Assigned( SelectedContact ) then
    begin
      Contact.Id := TContact( SelectedContact.Data^ ).Id;
      Contact.Name := TContact( SelectedContact.Data^ ).Name;
      Contact.Email := TContact( SelectedContact.Data^ ).Email;
      //Move( ,Contact,SizeOf( SelectedContact.Data^));
      ContactDblClicked := TContactDblClicked.Create( Contact );
      MsgManager.SendMessage( Self, ContactDblClicked );
    end;
end;

end.
